/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */
package org.netbeans.mdr.persistence.jdbcimpl;

import org.netbeans.mdr.persistence.*;
import java.sql.*;
import java.io.*;

/**
 * This is thrown to wrap an SQLException as a StorageException.
 *
 * @author John V. Sichi
 * @version $Id: JdbcStorageException.java,v 1.2 2004/03/07 03:27:01 jsichi Exp $
 *
 * @see StorageIOException
 */
public class JdbcStorageException extends StorageBadRequestException
{
    /**
     * The original SQLException that we are converting to a StorageException.
     */
    private SQLException sqlException;

    /** this constructs a JdbcStorageException from an SQLException
    * @param err the SQLException
    */
    public JdbcStorageException(SQLException err)
    {
        super(err.getMessage());
        sqlException = err;
    }

    /** return the original SQLException's message */
    public String getMessage()
    {
        return sqlException.getMessage();
    }
	
    /** return the original SQLException's localized message */
    public String getLocalizedMessage()
    {
        return sqlException.getLocalizedMessage();
    }

    /** print the original SQLException's stack trace */
    public void printStackTrace()
    {
        sqlException.printStackTrace();
    }

    /** print the original SQLException's stack trace */
    public void printStackTrace(PrintStream ps)
    {
        sqlException.printStackTrace(ps);
    }

    /** print the original SQLException's stack trace */
    public void printStackTrace(PrintWriter pw)
    {
        sqlException.printStackTrace(pw);
    }

    /** get the original SQLException's cause */
    public Throwable getCause()
    {
        return sqlException.getNextException();
    }
}

// End JdbcStorageException.java
