/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2003 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.mdrant;

import java.util.Iterator;

import org.apache.tools.ant.BuildException;

import javax.jmi.model.MofPackage;
import javax.jmi.model.ModelPackage;

/** Instantiates metamodel into given extent.
 *
 * @author  Petr Hrebejk
 */
public class Instantiate extends MdrTask.Sub {
    
    private String name;
    private String extent;
    private String packageName;
    
    /** Creates a new instance of PrintExtentsNames */
    public Instantiate() {
    }
    
    public void execute() throws Exception {
        
        if ( name == null ) {
            throw new BuildException( "Name for newly instantiated package has to be sppecified. Use the \"name\" attribute" );
        }
        
        if ( extent == null && packageName == null ) {
            getRepository().createExtent( name );
        }
        else {
            if ( extent == null ) {
                throw new BuildException( "Extent containing the package has to be specified. Use the \"extent\" attribute" );
            }
            if ( packageName == null ) {
                throw new BuildException( "The package has to be specified. Use the \"package\" attribute" );
            }
            
            MofPackage mp = getModelPackage();
            
            if ( mp == null ) {
                throw new BuildException( packageName + " does not exist in extent " + extent );
            }
            
            getRepository().createExtent( name, mp ); 
        }
        
    }
    
    // ANT task attributes -----------------------------------------------------
    
    public void setName( String name ) {
        this.name = name;
    }
    
    public void setExtent( String extent ) {
        this.extent = extent;
    }
    
    public void setPackage( String packageName ) {
        this.packageName = packageName;
    }
    
    // private methods ---------------------------------------------------------
    
    
    private MofPackage getModelPackage() {
        
        ModelPackage mofPackage = (ModelPackage)getRepository().getExtent( extent );
        MofPackage result;
        
        for (Iterator it = mofPackage.getMofPackage().refAllOfClass().iterator(); it.hasNext();) {
            result = (MofPackage) it.next();
            if (result.getName().equals( packageName )) {
                return result;
            }
        }
        
        return null;
    }
    
    
}
