/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2003 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.mdrant;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringWriter;

import javax.jmi.reflect.*;

import org.apache.tools.ant.BuildException;

import org.netbeans.api.mdr.*;
import org.netbeans.lib.jmi.mapping.FileStreamFactory;
import org.netbeans.lib.jmi.mapping.JMIMapperImpl;

/** Creates JMI maping in form of source files.
 *
 * @author  Petr Hrebejk
 */
public class MapJava extends MdrTask.Sub {
    
    private File dir; 
    private File headerFile;
    private String extent; 
    
    /** Creates a new instance of PrintExtentsNames */
    public MapJava() {
    }
    
    public void execute() throws Exception {
        
        if ( dir == null ) {
            throw new BuildException( "Destination directory has to be sppecified. Use the \"dir\" attribute" );
        }
        
        if ( extent == null ) {
            throw new BuildException( "Extent to map has to be sppecified. Use the \"extent\" attribute" );
        }
        
        JMIStreamFactory streamFactory = new FileStreamFactory( dir );
        
        getMapper().generate( streamFactory, getRepository().getExtent( extent ) );
        
    }
    
    // ANT task attributes -----------------------------------------------------
    
    public void setDir( File dir ) {
      this.dir = dir;
    }
    
    public void setExtent( String extent ) {
        this.extent = extent;
    }
    
    public void setHeaderFile(File headerFile) {
        this.headerFile = headerFile;
    }
    
    // Protected methods -------------------------------------------------------
    
    protected JMIMapper getMapper() throws IOException {
        JMIMapperImpl result = new JMIMapperImpl();
        if (headerFile != null) {
            BufferedReader reader = new BufferedReader(new FileReader(headerFile));
            StringWriter header = new StringWriter();
            int ch;
            while ((ch = reader.read()) != -1) {
                header.write(ch);
            }
            reader.close();
            header.close();
            result.setHeader(header.toString());
        }
        return result;
    }
    
    
}
